
/****************************************************************************
 * NCSA HDF                                                                 *
 * Scientific Data Technologies                                             *
 * National Center for Supercomputing Applications                          *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * hdf/COPYING f.                                                           *
 *                                                                          *
 ****************************************************************************/

#include <stdlib.h>

#include "H5HL.h"

#define FILE_NAME "lite_attr_test.h5"


#define ATTR1_NAME "attr string"
#define ATTR2_NAME "attr char"
#define ATTR3_NAME "attr short"
#define ATTR4_NAME "attr int"
#define ATTR5_NAME "attr long"
#define ATTR6_NAME "attr float"
#define ATTR7_NAME "attr double"

herr_t make_attributes( hid_t loc_id, const char* obj_name );



int main( void )
{

 hid_t   file_id;
 hid_t   dataset_id;
 hid_t   group_id;
 hid_t   space_id;  
 hsize_t dims[1] = { 5 };



 /* Create a new file using default properties. */
 file_id = H5Fcreate( FILE_NAME, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT );

/*-------------------------------------------------------------------------
 * Create a dataset named "dset" on the root group
 *-------------------------------------------------------------------------
 */

 /* Create the data space  */
 if ((space_id = H5Screate_simple( 1, dims, NULL ))<0) goto out;

 /* Create the dataset */
 if ((dataset_id = H5Dcreate( file_id , "dset", H5T_NATIVE_INT, space_id, 
  H5P_DEFAULT ))<0) goto out;

 /* Close */
 H5Dclose( dataset_id );

/*-------------------------------------------------------------------------
 * Create a group named "grp" on the root group
 *-------------------------------------------------------------------------
 */

 /* Create a group. */
 if ((group_id = H5Gcreate( file_id, "grp", 0 ))<0) goto out;
  
 /* Close */
 H5Gclose( group_id );


	
/*-------------------------------------------------------------------------
 * 
 * Create attributes in the root group
	* Note that we are calling the H5LTset_attribute functions with the name "." 
 * 
 *-------------------------------------------------------------------------
 */

 make_attributes( file_id, "." );

	
/*-------------------------------------------------------------------------
 * 
 * Create attributes in the dataset "dset"
 * 
 *-------------------------------------------------------------------------
 */

 make_attributes( file_id, "dset" );
	
/*-------------------------------------------------------------------------
 * 
 * Create attributes in the group "grp"
	*
 *-------------------------------------------------------------------------
 */

 make_attributes( file_id, "grp" );


/*-------------------------------------------------------------------------
 * end
 *-------------------------------------------------------------------------
 */

 /* Close the file. */
 H5Fclose( file_id );


 return 0;

out:
 /* Close the file. */
	H5Fclose( file_id );
 H5_FAILED();
 return 1;

 

}



herr_t make_attributes( hid_t loc_id, const char* obj_name )
{

 int         rank_out;
 hsize_t     *dims_out = 0;
 H5T_class_t type_class;
 size_t      type_size;
 int         i;

 char    attr_str_in[]     = {"My attribute"};
 char    attr_str_out[20];
 char    attr_char_in[5]   = {1,2,3,4,5};
 char    attr_char_out[5];
 short   attr_short_in[5]  = {1,2,3,4,5};
 short   attr_short_out[5];
 int     attr_int_in[5]    = {1,2,3,4,5};
 int     attr_int_out[5];
 long    attr_long_in[5]   = {1,2,3,4,5};
 long    attr_long_out[5];
 float   attr_float_in[5]  = {1,2,3,4,5};
 float   attr_float_out[5];
 double  attr_double_in[5] = {1,2,3,4,5};
 double  attr_double_out[5];

/*-------------------------------------------------------------------------
 * H5LTset_attribute_string test
 *-------------------------------------------------------------------------
 */

 TESTING("H5LTset_attribute_string");
 
 /* Set the attribute */
 if ( H5LTset_attribute_string( loc_id, obj_name, ATTR1_NAME, attr_str_in ) < 0 )
  return -1;

	PASSED();

/*-------------------------------------------------------------------------
 * H5LTset_attribute_string test
 *-------------------------------------------------------------------------
 */

	TESTING("H5LTget_attribute_string");

 
 /* Get the attribute */
 if ( H5LTget_attribute_string( loc_id, obj_name, ATTR1_NAME, attr_str_out ) < 0 )
  return -1;

 if ( strcmp( attr_str_in, attr_str_out ) != 0 ) 
 {
  return -1;
 }

	PASSED();


/*-------------------------------------------------------------------------
 * H5LTset_attribute_char test
 *-------------------------------------------------------------------------
 */

 TESTING("H5LTset_attribute_char");

 /* Set the attribute */
 if ( H5LTset_attribute_char( loc_id, obj_name, ATTR2_NAME, attr_char_in, 5 ) < 0 )
  return -1;

	PASSED();

/*-------------------------------------------------------------------------
 * H5LTget_attribute_char test
 *-------------------------------------------------------------------------
 */

	TESTING("H5LTget_attribute_char");

 /* Get the attribute */
 if ( H5LTget_attribute_char( loc_id, obj_name, ATTR2_NAME, attr_char_out ) < 0 )
  return -1;

 for (i = 0; i < 5; i++)
  {
   if ( attr_char_in[i] != attr_char_out[i] ) {
    return -1;
   }
  }

	/* Get the attribute */
 if ( H5LTget_attribute( loc_id, obj_name, ATTR2_NAME, H5T_NATIVE_CHAR, attr_char_out ) < 0 )
  return -1;

 for (i = 0; i < 5; i++)
  {
   if ( attr_char_in[i] != attr_char_out[i] ) {
    return -1;
   }
  }

 PASSED();

/*-------------------------------------------------------------------------
 * H5LTset_attribute_short test
 *-------------------------------------------------------------------------
 */

 TESTING("H5LTset_attribute_short");

 /* Set the attribute */
 if ( H5LTset_attribute_short( loc_id, obj_name, ATTR3_NAME, attr_short_in, 5 ) < 0 )
  return -1;

	PASSED();


/*-------------------------------------------------------------------------
 * H5LTget_attribute_short test
 *-------------------------------------------------------------------------
 */

 TESTING("H5LTget_attribute_short");

 /* Get the attribute */
 if ( H5LTget_attribute_short( loc_id, obj_name, ATTR3_NAME, attr_short_out ) < 0 )
  return -1;

 for (i = 0; i < 5; i++)
  {
   if ( attr_short_in[i] != attr_short_out[i] ) {
    return -1;
   }
  }

	/* Get the attribute */
 if ( H5LTget_attribute( loc_id, obj_name, ATTR3_NAME, H5T_NATIVE_SHORT, attr_short_out ) < 0 )
  return -1;

 for (i = 0; i < 5; i++)
  {
   if ( attr_short_in[i] != attr_short_out[i] ) {
    return -1;
   }
  }

 PASSED();


/*-------------------------------------------------------------------------
 * H5LTset_attribute_int test
 *-------------------------------------------------------------------------
 */

 TESTING("H5LTset_attribute_int");

 /* Set the attribute */
 if ( H5LTset_attribute_int( loc_id, obj_name, ATTR4_NAME, attr_int_in, 5 ) < 0 )
  return -1;

	PASSED();

/*-------------------------------------------------------------------------
 * H5LTget_attribute_int test
 *-------------------------------------------------------------------------
 */

 TESTING("H5LTget_attribute_int");

 /* Get the attribute */
 if ( H5LTget_attribute_int( loc_id, obj_name, ATTR4_NAME, attr_int_out ) < 0 )
  return -1;

 for (i = 0; i < 5; i++)
  {
   if ( attr_int_in[i] != attr_int_out[i] ) {
    return -1;
   }
  }

	/* Get the attribute */
 if ( H5LTget_attribute( loc_id, obj_name, ATTR4_NAME, H5T_NATIVE_INT, attr_int_out ) < 0 )
  return -1;

 for (i = 0; i < 5; i++)
  {
   if ( attr_int_in[i] != attr_int_out[i] ) {
    return -1;
   }
  }

 PASSED();

/*-------------------------------------------------------------------------
 * H5LTset_attribute_long test
 *-------------------------------------------------------------------------
 */

 TESTING("H5LTset_attribute_long");

 /* Set the attribute */
 if ( H5LTset_attribute_long( loc_id, obj_name, ATTR5_NAME, attr_long_in, 5 ) < 0 )
  return -1;

	PASSED();

/*-------------------------------------------------------------------------
 * H5LTget_attribute_long test
 *-------------------------------------------------------------------------
 */

 TESTING("H5LTget_attribute_long");

 /* Get the attribute */
 if ( H5LTget_attribute_long( loc_id, obj_name, ATTR5_NAME, attr_long_out ) < 0 )
  return -1;

 for (i = 0; i < 5; i++)
  {
   if ( attr_long_in[i] != attr_long_out[i] ) {
    return -1;
   }
  }

	/* Get the attribute */
 if ( H5LTget_attribute( loc_id, obj_name, ATTR5_NAME, H5T_NATIVE_LONG, attr_long_out ) < 0 )
  return -1;

 for (i = 0; i < 5; i++)
  {
   if ( attr_long_in[i] != attr_long_out[i] ) {
    return -1;
   }
  }

 PASSED();

/*-------------------------------------------------------------------------
 * H5LTset_attribute_float test
 *-------------------------------------------------------------------------
 */

 TESTING("H5LTset_attribute_float");

 /* Set the attribute */
 if ( H5LTset_attribute_float( loc_id, obj_name, ATTR6_NAME, attr_float_in, 5 ) < 0 )
  return -1;

	PASSED();

/*-------------------------------------------------------------------------
 * H5LTget_attribute_float test
 *-------------------------------------------------------------------------
 */

 TESTING("H5LTget_attribute_float");


 /* Get the attribute */
 if ( H5LTget_attribute_float( loc_id, obj_name, ATTR6_NAME, attr_float_out ) < 0 )
  return -1;

 for (i = 0; i < 5; i++)
  {
   if ( attr_float_in[i] != attr_float_out[i] ) {
    return -1;
   }
  }

	/* Get the attribute */
 if ( H5LTget_attribute( loc_id, obj_name, ATTR6_NAME, H5T_NATIVE_FLOAT, attr_float_out ) < 0 )
  return -1;

 for (i = 0; i < 5; i++)
  {
   if ( attr_float_in[i] != attr_float_out[i] ) {
    return -1;
   }
  }

 PASSED();

/*-------------------------------------------------------------------------
 * H5LTset_attribute_double test
 *-------------------------------------------------------------------------
 */

 TESTING("H5LTset_attribute_double");

 /* Set the attribute */
 if ( H5LTset_attribute_double( loc_id, obj_name, ATTR7_NAME, attr_double_in, 5 ) < 0 )
  return -1;

	PASSED();

/*-------------------------------------------------------------------------
 * H5LTget_attribute_double test
 *-------------------------------------------------------------------------
 */

 TESTING("H5LTget_attribute_double");

 /* Get the attribute */
 if ( H5LTget_attribute_double( loc_id, obj_name, ATTR7_NAME, attr_double_out ) < 0 )
  return -1;

 for (i = 0; i < 5; i++)
  {
   if ( attr_double_in[i] != attr_double_out[i] ) {
    return -1;
   }
  }

 /* Get the attribute */
 if ( H5LTget_attribute( loc_id, obj_name, ATTR7_NAME, H5T_NATIVE_DOUBLE, attr_double_out ) < 0 )
  return -1;

 for (i = 0; i < 5; i++)
  {
   if ( attr_double_in[i] != attr_double_out[i] ) {
    return -1;
   }
  }

 PASSED();


/*-------------------------------------------------------------------------
 * H5LTget_attribute_ndims test 
 *-------------------------------------------------------------------------
 */


 TESTING("H5LTget_attribute_ndims");
 
 if ( H5LTget_attribute_ndims( loc_id, obj_name, ATTR2_NAME, &rank_out ) < 0 )
  return -1;

 if ( rank_out != 1 ) {
  return -1;
 }

 PASSED();

/*-------------------------------------------------------------------------
 * H5LTget_attribute_info test 
 *-------------------------------------------------------------------------
 */

 TESTING("H5LTget_attribute_info");

 dims_out = malloc( sizeof(hsize_t) * rank_out );

 if ( H5LTget_attribute_info( loc_id, obj_name, ATTR2_NAME, dims_out, &type_class,
       &type_size) < 0 )
  return -1;

  for (i = 0; i < rank_out; i++)
  {
   if ( dims_out[i] != 5 ) {
    return -1;
   }
  }

 if ( type_class != H5T_INTEGER ) {
   return -1;
  }

 if ( dims_out )
  free( dims_out );

  PASSED();


 return 0;

}



